@@ -1,6 +1,6 @@
 package Mojo::Webqq;
 use strict;
-$Mojo::Webqq::VERSION = "1.4.5";
+$Mojo::Webqq::VERSION = "1.4.6";
 use base qw(Mojo::Base);
 use Mojo::Webqq::Log;
 use Mojo::Webqq::Cache;
@@ -17,6 +17,7 @@ has log_level               => 'info';     #debug|info|warn|error|fatal
 has log_path                => undef;
 has email                   => undef;
 has encrypt_method          => "perl";     #perl|js
+has friend_pic_dir          => undef;
 has ioloop                  => sub{Mojo::IOLoop->singleton};
 
 has version                 => $Mojo::Webqq::VERSION;
@@ -71,6 +72,7 @@ has ua                      => sub {
     local $ENV{MOJO_USERAGENT_DEBUG} = $_[0]->ua_debug;
     require Mojo::UserAgent;
     Mojo::UserAgent->new(
+        max_redirects      => 7,
         request_timeout    => 30,
         inactivity_timeout => 30,
         transactor => Mojo::UserAgent::Transactor->new( 
