@@ -16,6 +16,10 @@ has [qw(
     client_type 
 )];
 
+has displayname => sub{
+    my $self = shift;
+    return defined $self->markname?$self->markname:$self->nick;
+};
 has qq => sub{
     my $self = shift;
     return $self->{_client}?$self->{_client}->get_qq_from_id($self->id):undef;
